<?php

namespace FM\Service;

use FM\Factory\MinhasTarefasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;
use FM\Service\FileServerService;

class MinhasTarefasService
{

    protected $MinhasTarefasFactory;
    protected $FuncionariosHierarqService;
    protected $FileServerService;
    protected $FindService;
    protected $environment;
    protected $newCode;
    protected $fileServerUrl;

    public function __construct(EntityManager $entityManager, MinhasTarefasFactory $MinhasTarefasFactory, FileServerService $FileServerService, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $environment, NewCode $newCode, $fileServerUrl)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->MinhasTarefasFactory         = $MinhasTarefasFactory;
        $this->FileServerService            = $FileServerService;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->fileServerUrl                = $fileServerUrl;
    }

    public function getFotosTarefas($parametros)
    {
        return $this->MinhasTarefasFactory->getFotosTarefas(array(
            "NRCHECKLISTAGEND" => $parametros['NRCHECKLISTAGEND']
        ));
    }

    public function excluirFotoTarefas($parametros)
    {
        return $this->MinhasTarefasFactory->excluirFotoTarefas(array(
            "NRFOTOTAREFAAGEND" => $parametros['NRFOTOTAREFAAGEND']
        ));
    }

    public function getTarefasListadas($parametros)
    {
        $filtro = '';
        $cdOperador = $this->environment->getCdOperador();
        $retorno = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdOperador);


        if ($parametros['TIPO'] == "1") {
            $filtro = $filtro . " AND CA.IDATIVO = 'S'";
        } elseif ($parametros['TIPO'] == "2") {
            if ($parametros['NRCHECKLISTAGEND'] != '' && $parametros['NRCHECKLISTAGEND'] != null) {
                $filtro = $filtro . " AND CA.NRCHECKLISTAGEND = " . $parametros['NRCHECKLISTAGEND'];
            }
        } elseif ($parametros['TIPO'] == "3") {
            if (isset($parametros['NRSEQCONTRATO'])) {
                $contratos = $this->FindService->formatarArray($parametros['NRSEQCONTRATO']);
                $filtro = $filtro . " AND CA.NRSEQCONTRATO IN ( " . $contratos . ")";
            }

            if (isset($parametros['CDCLIENTE'])) {
                $clientes = $this->FindService->formatarArray($parametros['CDCLIENTE']);
                $filtro = $filtro . " AND LC.CDCLIENTE IN ( " . $clientes . ")";
            }

            if (isset($parametros['NRLOCALCLIENTE'])) {
                $local = $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']);
                $filtro = $filtro . " AND CA.NRLOCALCLIENTE IN ( " . $local . ")";
            }

            if (isset($parametros['CDOPERADOR'])) {
                $operadores = $this->FindService->formatarArray($parametros['CDOPERADOR']);
                $filtro = $filtro . " AND CA.CDOPERADOR IN ( " . $operadores . ")";
            }

            if (isset($parametros['IDSITUACAOAGEND'])) {
                $situacao = $this->FindService->formatarArray($parametros['IDSITUACAOAGEND']);
                $filtro = $filtro . " AND CA.IDSITUACAOAGEND IN ( " . $situacao . ")";
            }

            if (isset($parametros['DTHORAGEND'])) {
                $dataInicial = $parametros['DTHORAGEND'][0];
                $dataFinal = array_key_exists("1", $parametros['DTHORAGEND']) ? $parametros['DTHORAGEND'][1] : null;

                if ($dataFinal != null) {
                    $filtro = $filtro . " AND CA.DTHORAAGEND BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
                } else {
                    $data = $dataInicial;
                    $filtro = $filtro . " AND CA.DTHORAAGEND BETWEEN '" . $data . "' AND TO_DATE('" . $data . "')+30";
                }
            }

            if (isset($parametros['IDATIVO'])) {
                $filtro = $filtro . " AND CA.IDATIVO = '" . $parametros['IDATIVO']."'";
            }
        }

        return $this->MinhasTarefasFactory->getTarefasListadas($retorno, $filtro);
    }

    // Funções da Tela Minhas Tarefas
    public function anexarImagensTarefas($parametros)
    {
        $this->connection->beginTransaction();
       
        try {
            $NRORG = $this->environment->getNrOrg();
            $cdOperador = $this->environment->getCdOperador();

            $codTarefaAgend = $this->MinhasTarefasFactory->getCodTarefaAgend(array(
                "NRCHECKLISTAGEND" => $parametros['NRCHECKLISTAGEND'],
            ));

            $imagens = $this->FileServerService->uploadFiles($parametros['IMAGENS'], 'anexosTarefa');
        
            foreach ($imagens as $img) {
                $this->MinhasTarefasFactory->anexarImagemTarefas(array(
                    "NRFOTOTAREFAAGEND" => $this->newCode->getCodeByTableName("FCM_FOTOTAREFAAGEND", $NRORG, 1, 10, true),
                    "NRTAREFAAGEND"     => $codTarefaAgend[0]['NRTAREFAAGEND'],
                    "DSFOTOTAREFAAGEND" => $this->fileServerUrl . '/files/' . $img['path'],
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $cdOperador,
                    "NRORGINCLUSAO"     => $NRORG,
                    "DSTYPE"            => $img['type'],
                    "NRSIZE"            => $img['size']
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getMinhasTarefas($parametros)
    {
        $filtro = '';
        $operadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->environment->getCdOperador());

        if ($parametros['TIPO'] == '2') {
            if (isset($parametros['NRCHECKLISTAGEND'])) {
                $tarefas = $this->FindService->formatarArray($parametros['NRCHECKLISTAGEND']);
                $filtro = $filtro . " AND CA.NRCHECKLISTAGEND IN ( " . $tarefas . ")";
            }

            if (isset($parametros['NRSEQCONTRATO'])) {
                $contratos = $this->FindService->formatarArray($parametros['NRSEQCONTRATO']);
                $filtro = $filtro . " AND CA.NRSEQCONTRATO IN ( " . $contratos . ")";
            }

            if (isset($parametros['CDCLIENTE'])) {
                $clientes = $this->FindService->formatarArray($parametros['CDCLIENTE']);
                $filtro = $filtro . " AND LC.CDCLIENTE IN ( " . $clientes . ")";
            }

            if (isset($parametros['NRLOCALCLIENTE'])) {
                $local = $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']);
                $filtro = $filtro . " AND CA.NRLOCALCLIENTE IN ( " . $local . ")";
            }

            if (isset($parametros['DTHORAGEND'])) {
                $dataInicial = $parametros['DTHORAGEND'][0];
                $dataFinal = array_key_exists("1", $parametros['DTHORAGEND']) ? $parametros['DTHORAGEND'][1] : null;

                if ($dataFinal != null) {
                    $filtro = $filtro . " AND CA.DTHORAAGEND BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
                } else {
                    $data = $dataInicial;
                    $filtro = $filtro . " AND CA.DTHORAAGEND BETWEEN '" . $data . "' AND TO_DATE('" . $data . "')+30";
                }
            }
        }

        return $this->MinhasTarefasFactory->getMinhasTarefas($operadores, $filtro);
    }

    public function alterarStatusItem($params)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->environment->getCdOperador();
            $nrOrg = $this->environment->getNrOrg();

            if (!empty($params['MARCADOS'])) {
                $this->MinhasTarefasFactory->alterarStatusItens(array(
                    "NRCHECKLISTAGEND"  => $params['NRCHECKLISTAGEND'],
                    "CDOPERULTATU"      => $cdOperador,
                    "NRORGULTATU"       => $nrOrg,
                    "IDREALIZADO"       => 'S'
                ), $this->FindService->formatarArray($params['MARCADOS']));
            }

            if (!empty($params['NMARCADOS'])) {
                $this->MinhasTarefasFactory->alterarStatusItens(array(
                    "NRCHECKLISTAGEND"  => $params['NRCHECKLISTAGEND'],
                    "CDOPERULTATU"      => $cdOperador,
                    "NRORGULTATU"       => $nrOrg,
                    "IDREALIZADO"       => 'N'
                ), $this->FindService->formatarArray($params['NMARCADOS']));
            }
                
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTarefasPorOperador()
    {
        $operadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->environment->getCdOperador());
        return $this->MinhasTarefasFactory->getTarefasPorOperador($operadores);
    }

    public function alterarStatusTarefa($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->environment->getCdOperador();
            $nrOrg = $this->environment->getNrOrg();

            $param = array(
                "IDSITUACAOAGEND"    => $parametros['IDSITUACAOAGEND'],
                "DSJUSTCANCELAMENTO" => (isset($parametros["DSJUSTCANCELAMENTO"])) ? $parametros["DSJUSTCANCELAMENTO"] : null,
                "NRCHECKLISTAGEND"   => $parametros['NRCHECKLISTAGEND'],
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $nrOrg
            );

            $this->MinhasTarefasFactory->alterarStatusTarefa($param);

            if ($parametros['TIPO'] == 'reportar') {
                $param = array(
                    "IDSITUACAOGERCHK"  => $parametros['IDSITUACAOAGEND'],
                    "NRCHECKLISTAGEND"  => $parametros['NRCHECKLISTAGEND'],
                    "CDOPERULTATU"      => $cdOperador,
                    "NRORGULTATU"       => $nrOrg
                );

                $this->MinhasTarefasFactory->upStatusVistoryCanc($param);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarObservacao($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->environment->getCdOperador();
            $nrOrg = $this->environment->getNrOrg();

            $param = array(
                "NRCHECKLISTAGEND"   => $parametros['NRCHECKLISTAGEND'],
                "DSOBSTAREFAAGEND"  => $parametros['DSOBSTAREFAAGEND'],
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $nrOrg
            );

            $this->MinhasTarefasFactory->salvarObservacao($param);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    // Funções da Tela de Detalhamento
    public function getDetalhesTarefa($params)
    {
        return $this->MinhasTarefasFactory->getDetalhesTarefa(array(
            "NRCHECKLISTAGEND"    => $params['NRCHECKLISTAGEND'],
        ));
    }

    public function getItensTarefas($params)
    {
        return $this->MinhasTarefasFactory->getItensTarefas(array(
            "NRCHECKLISTAGEND"    => $params['NRCHECKLISTAGEND'],
        ));
    }

    public function qtdTarefasAtivas()
    {
        $operadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->environment->getCdOperador());
        return $this->MinhasTarefasFactory->qtdTarefasAtivas($operadores);
    }

    public function statusCheck()
    {
        return $this->MinhasTarefasFactory->statusCheck(array(
            "CDOPERADOR" => $this->environment->getCdOperador()
        ));
    }

    public function finalizarTarefa($params)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->environment->getCdOperador();
            $nrOrg = $this->environment->getNrOrg();

            $parametros = array(
                "IDSITUACAOAGEND"    => 4,
                "DSJUSTCANCELAMENTO" => null,
                "NRCHECKLISTAGEND"   => $params['NRCHECKLISTAGEND'],
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $nrOrg
            );

            $this->MinhasTarefasFactory->alterarStatusTarefa($parametros);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function itensDetailsTaskForVistoria($params)
    {
        $parametros = array(
            "NRGERENCIAMENTOCHK" => $params['NR'],
        );
        return $this->MinhasTarefasFactory->itensDetailsTaskForVistoria($parametros);
    }

    // ------------------------------ FUNÇÔES GEOLOCALIZAÇÃO ------------------------------
    public function getGeolocalizacao($param)
    {
        return $this->MinhasTarefasFactory->getGeolocalizacao($param['NRLOCALCLIENTE']);
    }

    public function getParametrizacao()
    {
        return $this->MinhasTarefasFactory->getParametrizacao();
    }
}

