import { Utilities } from '@/util/Utilities';
import {
    DateRange, Form, LoadingService, Select, TextInput, Text,
} from '@zeedhi/common';
import { Http, Loader, Metadata } from '@zeedhi/core';

export class ParametrizacaoController {
    private tarefaOffline = Loader.getInstance('Fm00001TarefasControllerOffline');

    public tipoCarregamento: String = '';

    public text = '';

    public visibilidadeTab = false;

    public qtdDias: any;

    public habilitaOffline: any;

    public tpOffline: any;

    public dtFinalInicial: any;

    public syncDadosBackEnd() {
        this.tarefaOffline.prepararDadosIndexDb();
    }

    public setaValorDias() {
        const xDias = Number(Metadata.getInstance<TextInput>('qtdDiasParam').value);
        const diaAtual = new Date();
        const diaFinal = new Date(diaAtual.getFullYear(), diaAtual.getMonth(), diaAtual.getDate() + xDias);
        this.text = `Período escolhido: ${diaAtual.getDate()}/${diaAtual.getMonth() + 1}/${diaAtual.getFullYear()} até ${diaFinal.getDate()}/${diaFinal.getMonth() + 1}/${diaFinal.getFullYear()}`;
    }

    public buscarDadosBackEnd() {
        LoadingService.show();
        this.tarefaOffline.atualizarBancoParam = true;
        console.warn(this.tarefaOffline.atualizarBancoParam);
        this.tarefaOffline.inicializarTabelas();
    }

    public deletarBancoDados() {
        const req = indexedDB.deleteDatabase(`Tarefa_${window.localStorage.getItem('CDOPERADOR')}`);
        req.onsuccess = () => {
            console.warn('Deleted database successfully');
        };
        req.onerror = () => {
            console.warn('Couldnt delete database');
        };
        req.onblocked = () => {
            console.warn('Couldnt delete database due to the operation being blocked');
        };
        window.location.reload();
    }

    public changeTipoCarregamento({ component }: any) {
        const xDias = Metadata.getInstance<TextInput>('qtdDiasParam');
        const personalizado = Metadata.getInstance<DateRange>('dataParam');
        const exibirDatas = Metadata.getInstance<Text>('textAreaDias');
        if (component.value === '1') {
            xDias.isVisible = true;
            personalizado.isVisible = false;
            personalizado.value = null;
            exibirDatas.isVisible = true;
        } else {
            personalizado.isVisible = true;
            xDias.isVisible = false;
            xDias.value = null;
            exibirDatas.isVisible = false;
        }
        this.tipoCarregamento = component.value;
    }

    public salvarPrametrizacoes() {
        const xDias = Metadata.getInstance<TextInput>('qtdDiasParam').value;
        const personalizado = Metadata.getInstance<DateRange>('dataParam').value;
        const modoOffline = Metadata.getInstance<Select>('selectSimpleResult').value;
        const form = Metadata.getInstance<Form>('form-filtros');
        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/parametrizacaoOfflineApp', {
                XDIAS: xDias,
                DATARANGE: personalizado,
                TIPOCARREGAMENTO: this.tipoCarregamento,
                USAOFFLINE: modoOffline,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public buscarInformacoesParam() {
        LoadingService.show();
        Http.get('/getParamOffline').then((response) => {
            this.qtdDias = response.data[0].QTDIASCARREGAOFFLINE;
            this.habilitaOffline = response.data[0].IDHABILITAOFFLINE;
            this.tpOffline = response.data[0].IDTIPOCARREGAOFFLINE;
            this.dtFinalInicial = [response.data[0].DTINICAL];
            this.dtFinalInicial.push([response.data[0].DTFINAL]);
            if (response.data[1].length > 14) {
                this.visibilidadeTab = true;
            } else {
                this.visibilidadeTab = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public configurarParam() {
        Metadata.getInstance<TextInput>('qtdDiasParam').value = this.qtdDias;
        Metadata.getInstance<Select>('selectSimpleResult').value = this.habilitaOffline;
        Metadata.getInstance<Select>('defineEscolhaData').value = this.tpOffline;
        Metadata.getInstance<DateRange>('dataParam').value = this.dtFinalInicial;
    }
}
